using FitConnect;
using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;

namespace E2ETest;

[Order(20)]
[TestFixture]
public class StraightForwardTest : EndToEndTestBase {
    private string _caseId = null!;
    private string _submissionId = null!;

    [Order(10)]
    [Test]
    public void SendingSubmission() {
        var outgoing = SubmissionBuilder
            .WithDestination(Settings.DestinationId)
            .WithServiceType("Straight forward test", Settings.LeikaKey)
            .WithJsonData(@"{""data"":""value""}")
            .WithAttachments(new Attachment("Test.pdf", "Attachment #1"),
                new Attachment("Test.pdf", "Attachment #2"))
            .Build();

        var submission = Sender.SendAsync(outgoing).Result;

        _caseId = submission.CaseId!;
        _submissionId = submission.Id!;

        _caseId.Should().NotBeNull();
        _submissionId.Should().NotBeNull();
        submission.GeneratedMetadata.Should().StartWith("{\"$schema\"");
    }

    [Order(11)]
    [Test]
    public void SendingSubmission_WithoutAttachments() {
        var outgoing = SubmissionBuilder.WithDestination(Settings.DestinationId)
            .WithServiceType("Straight forward test", Settings.LeikaKey)
            .WithJsonData(@"{""data"":""value""}")
            .Build();

        var submission = Sender.SendAsync(outgoing).Result;

        var caseId = submission.CaseId!;
        var submissionId = submission.Id!;

        caseId.Should().NotBeNull();
        submissionId.Should().NotBeNull();
    }

    [Test]
    [Order(20)]
    public void Sender_GetSubmissionState() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_caseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetStatusForSubmissionAsync(_caseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void RequestSubmission() {
        var subscriberWithSubmission = Subscriber.RequestSubmissionAsync(_submissionId).Result;

        var data = subscriberWithSubmission.GetDataString();
        Logger.LogInformation("Data {Data}", data);

        data.Should().Be(@"{""data"":""value""}");

        var attachments = subscriberWithSubmission.GetAttachmentsAsync().Result.ToList();
        attachments.First().Filename.Should().Be("Test.pdf");
        attachments.TrueForAll(a => a.Description?.StartsWith("Attachment #") ?? false).Should()
            .BeTrue();

        subscriberWithSubmission.AcceptSubmissionAsync().Wait();
    }

    [Test]
    [Order(50)]
    public void Sender_GetSubmissionState_AfterAccepting() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_caseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.Any(s => s.EventType == EventType.Accept);

        var submitStatus = status.Single(s => s.EventType == EventType.Submit);
        submitStatus.Events?.SubmitSubmissionEvent?.AuthenticationTags?.Attachments.Should()
            .NotBeNullOrEmpty();

        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }
}
